<?php

namespace Drupal\gtext\Form;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\AppendCommand;
use Drupal\Core\Ajax\CloseDialogCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormStateInterface;
use Google\Cloud\Translate\V2\TranslateClient;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Form\ConfigFormBase;

/**
 * Provides a SettingsForm class for forms.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Defines the configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactory $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('config.factory'),
      );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'form_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'gtext.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = ['#id' => 'form__settings_form'];

    $form['api_key'] = [
      '#type'           => 'textfield',
      '#title'          => gtext()->gtext('Google API key'),
      '#description'    => gtext()->gtext('Please enter your Google API key or visit <a href="@url">Google APIs console</a> to create new one.', ['@url' => 'https://code.google.com/apis/console']),
      '#default_value'  => $this->configFactory->get('gtext.settings')->get('google_api_key', ''),
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type'        => 'submit',
      '#value'       => gtext()->gtext('Save'),
      '#button_type' => 'primary',
      '#ajax'        => [
        'wrapper'      => $form['#id'],
        'callback'     => __CLASS__ . '::ajaxCallback',
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $key = $form_state->getValue(['api_key']);
    if (!empty($key)) {
      try {
        $client = new TranslateClient(
              [
                'key' => $key,
              ]
          );
      }
      catch (\Exception $e) {
        $message = @json_decode($e->getMessage(), TRUE);
        if ($message !== FALSE && isset($message['error']['code']) && isset($message['error']['message'])) {
          $message = '[' . $message['error']['code'] . '] ' . $message['error']['message'];
        }
        else {
          $message = $e->getMessage();
        }
        $form_state->setError($form['api_key'], $message);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->configFactory->getEditable('gtext.settings')
      ->set('google_api_key', $form_state->getValue(['api_key']))
      ->save();
    $this->messenger()->addStatus(gtext()->gtext('API key saved'));
  }

  /**
   * Ajax callback.
   */
  public static function ajaxCallback(array &$form, FormStateInterface $form_state) {
    if ($form_state->hasAnyErrors() || !$form_state->isSubmitted()) {
      return $form;
    }
    $response = new AjaxResponse();
    $status = TranslateFilterForm::getApiStatus();
    $response->addCommand(new ReplaceCommand('#' . $status['#attributes']['id'], $status));
    $response->addCommand(new CloseDialogCommand());
    $response->addCommand(new AppendCommand('body', ['#type' => 'status_messages']));
    return $response;
  }

}
